@@ -31,3 +31,17 @@ class OutOfGasError(Exception):
 
     """
     pass
+
+
+class AlreadyFillingGasError(Exception):
+    """Exception raised when additional gas refills are issued while one is still in progress
+
+    """
+    pass
+
+
+class InitializationError(Exception):
+    """Exception raised when initialization state is insufficient to run component
+
+    """
+    pass
