@@ -7,7 +7,7 @@ import celery
 import web3
 
 # platform imports
-from cic_registry import ContractRegistry
+from cic_registry import CICRegistry
 from cic_eth.db import SessionBase
 from cic_eth.db.models.convert import TxConvertTransfer
 from cic_eth.db.models.otx import Otx
@@ -34,12 +34,12 @@ class BancorTxFactory(TxFactory):
     local_inited = False
 
     @staticmethod
-    def init(w3, from_address):
-        TxFactory.init(w3, from_address)
+    def init(w3, chain_spec): #, from_address):
+        TxFactory.init(w3, chain_spec) # , from_address)
         if BancorTxFactory.local_inited:
             logg.debug('bancortxfactory already inited, skipping')
             return
-        BancorTxFactory.network_contract = ContractRegistry.get_contract('BancorNetwork')
+        BancorTxFactory.network_contract = CICRegistry.get_contract(chain_spec, 'BancorNetwork')
         BancorTxFactory.local_inited = True
 
 
@@ -88,7 +88,7 @@ class BancorTxFactory(TxFactory):
                 'from': self.address,
                 'gas': network_gas,
                 'gasPrice': self.gas_price,
-                'chainId': ContractRegistry.chain_id,
+                'chainId': CICRegistry.chain_spec.chain_id(),
                 'nonce': self.next_nonce(),
                 })
         return tx_convert
@@ -140,27 +140,6 @@ def unpack_convert(data):
 #        }
 
 
-@celery_app.task()
-def resolve_tokens_by_symbol(token_symbols):
-    """Returns contract addresses of an array of ERC20 token symbols
-
-    :param token_symbols: Token symbols to resolve
-    :type token_symbols: list of str
-
-    :return: Respective token contract addresses
-    :rtype: list of str, 0x-hex
-    """
-    #cr = ContractRegistry.get_contract('ConverterRegistry').contract
-    tokens = []
-    for token_symbol in token_symbols:
-        token = ContractRegistry.get_token_by_symbol(token_symbol)
-        tokens.append({
-            'address': token.address,
-            'converters': [],
-            })
-    return tokens
-
-
 @celery_app.task(bind=True)
 def convert_with_default_reserve(self, tokens, from_address, source_amount, minimum_return, to_address=None):
     """Performs a conversion between two liquid tokens using Bancor network.
@@ -178,21 +157,24 @@ def convert_with_default_reserve(self, tokens, from_address, source_amount, mini
     queue = self.request.delivery_info['routing_key']
 
     c = RpcClient(from_address)
-    BancorTxFactory.init(c.w3, from_address)
 
-    cr = ContractRegistry.get_contract('BancorNetwork')
-    source_token = ContractRegistry.addresses[tokens[0]['address']].contract
-    reserve_address = ContractRegistry.contracts['BNTToken'].address
+    chain_spec = CICRegistry.chain_spec
 
-    tx_factory = TokenTxFactory()
+    cr = CICRegistry.get_contract(chain_spec, 'BancorNetwork')
+    source_token = CICRegistry.get_address(chain_spec, tokens[0]['address'])
+    reserve_address = CICRegistry.get_contract(chain_spec, 'BNTToken').address()
+
+    TxFactory.init(c.w3, chain_spec) #, from_address)
+    tx_factory = TokenTxFactory(from_address)
     
-    tx_approve_zero = tx_factory.approve(source_token.address, cr.address, 0)
+    tx_approve_zero = tx_factory.approve(source_token.address(), cr.address(), 0)
     (tx_approve_zero_hash_hex, tx_approve_zero_signed_hex) = sign_and_register_tx(tx_approve_zero) 
 
-    tx_approve = tx_factory.approve(source_token.address, cr.address, source_amount)
+    tx_approve = tx_factory.approve(source_token.address(), cr.address(), source_amount)
     (tx_approve_hash_hex, tx_approve_signed_hex) = sign_and_register_tx(tx_approve) 
 
-    tx_factory = BancorTxFactory()
+    BancorTxFactory.init(c.w3, chain_spec) #, from_address)
+    tx_factory = BancorTxFactory(from_address)
     tx_convert = tx_factory.convert(
             tokens[0]['address'],
             tokens[1]['address'],
@@ -206,10 +188,10 @@ def convert_with_default_reserve(self, tokens, from_address, source_amount, mini
         save_convert_recipient(tx_convert_hash_hex, to_address)
 
     s = create_check_gas_and_send_task(
-            [tx_approve_hash_hex],
             [tx_approve_zero_signed_hex, tx_approve_signed_hex, tx_convert_signed_hex],
             from_address,
             tx_approve_zero['gasPrice'] * tx_approve_zero['gas'],
+            [tx_approve_hash_hex],
             queue,
             )
     s.apply_async()
@@ -304,8 +286,10 @@ def resolve_converters_by_tokens(tokens):
     :return: Addresses of matching converters
     :rtype: list of str, 0x-hex
     """
+    chain_spec = CICRegistry.chain_spec
     for t in tokens:
-        fn = ContractRegistry.function('ConverterRegistry', 'getConvertersByAnchors')
+        c = CICRegistry.get_contract(chain_spec, 'ConverterRegistry')
+        fn = c.function('getConvertersByAnchors')
         try:
             converters = fn([t['address']]).call()
         except Exception as e:
@@ -339,12 +323,13 @@ def transfer_converted(self, tokens, holder_address, receiver_address, value, tx
 
     queue = self.request.delivery_info['routing_key']
 
+    chain_spec = CICRegistry.chain_spec
     c = RpcClient(holder_address)
 
     # get transaction parameters
     gas_price = c.gas_price()
-    TokenTxFactory.init(c.w3, holder_address)
-    tx_factory = TokenTxFactory()
+    TokenTxFactory.init(c.w3, chain_spec) #, holder_address)
+    tx_factory = TokenTxFactory(holder_address)
 
     token_address = tokens[0]['address']
     tx_transfer = tx_factory.transfer(
@@ -357,10 +342,10 @@ def transfer_converted(self, tokens, holder_address, receiver_address, value, tx
     # send transaction
     logg.info('transfer converted token {} from {} to {} value {} {}'.format(token_address, holder_address, receiver_address, value, tx_transfer_signed_hex))
     s = create_check_gas_and_send_task(
-            [tx_transfer_hash_hex],
             [tx_transfer_signed_hex],
             holder_address,
             tx_transfer['gasPrice'] * tx_transfer['gas'],
+            None,
             queue,
             )
     s_save = celery.signature(
@@ -382,8 +367,9 @@ def otx_cache_convert(
         tx_signed_raw_hex,
         ):
 
+    chain_spec = CICRegistry.chain_spec
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, ContractRegistry.chain_id)
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, chain_spec.chain_id())
     tx_data = unpack_convert(tx['data'])
     logg.debug('tx data {}'.format(tx_data))
 
