@@ -12,7 +12,7 @@ import web3
 import logging
 
 # platform imports
-from cic_registry import ContractRegistry
+from cic_registry import CICRegistry
 from cic_eth.db.models.tx import TxCache
 from cic_eth.db.models.base import SessionBase
 from cic_eth.eth import RpcClient
@@ -49,7 +49,8 @@ class TokenTxFactory(TxFactory):
         :returns: Unsigned "approve" transaction in standard Ethereum format
         :rtype: dict
         """
-        source_token = ContractRegistry.get_address(token_address)
+        chain_spec = CICRegistry.chain_spec
+        source_token = CICRegistry.get_address(chain_spec, token_address)
         source_token_contract = source_token.contract
         tx_approve_buildable = source_token_contract.functions.approve(
             spender_address,
@@ -61,7 +62,7 @@ class TokenTxFactory(TxFactory):
             'from': self.address,
             'gas': source_token_gas,
             'gasPrice': self.gas_price,
-            'chainId': ContractRegistry.chain_id,
+            'chainId': chain_spec.chain_id(),
             'nonce': self.next_nonce(),
             })
         return tx_approve
@@ -84,7 +85,8 @@ class TokenTxFactory(TxFactory):
         :returns: Unsigned "transfer" transaction in standard Ethereum format
         :rtype: dict
         """
-        source_token = ContractRegistry.get_address(token_address)
+        chain_spec = CICRegistry.chain_spec
+        source_token = CICRegistry.get_address(chain_spec, token_address)
         source_token_contract = source_token.contract
         transfer_buildable = source_token_contract.functions.transfer(
                 receiver_address,
@@ -115,25 +117,6 @@ def unpack_transfer(data):
         }
 
 
-@celery_app.task()
-def add_token_to_registry(token_address):
-    """Adds a ERC20 token to in-memory registry
-
-    :param token_address: Address of token to add to registry
-    :type token_address: str, 0x-hex
-    :return: address: token address, converters: converter address
-    :rtype: dict
-    """
-    c = RpcClient()
-    t = ContractRegistry.add_token(c.w3, token_address)
-    return {
-        'address': t.address,
-        'converters': [
-            t.converter_address,
-            ]
-            }
-
-
 @celery_app.task()
 def balance(tokens, holder_address):
     """Return token balances for a list of tokens for given address
@@ -145,11 +128,14 @@ def balance(tokens, holder_address):
     :return: List of balances
     :rtype: list of int
     """
-    abi = ContractRegistry.abi('ERC20Token')
+    #abi = ContractRegistry.abi('ERC20Token')
     balances = []
     c = RpcClient()
     for t in tokens:
-        o = c.w3.eth.contract(abi=abi, address=t['address'])
+        #token = CICRegistry.get_address(t['address'])
+        #abi = token.abi()
+        #o = c.w3.eth.contract(abi=abi, address=t['address'])
+        o = CICRegistry.get_address(CICRegistry.chain_spec, t['address']).contract
         b = o.functions.balanceOf(holder_address).call()
         logg.debug('balance {} for {}: {}'.format(t['address'], holder_address, b))
         balances.append(b)
@@ -182,13 +168,13 @@ def transfer(self, tokens, holder_address, receiver_address, value):
     queue = self.request.delivery_info['routing_key']
 
     # retrieve the token interface
-    abi = ContractRegistry.abi('ERC20Token')
     t = tokens[0]
 
     c = RpcClient(holder_address)
 
-    TokenTxFactory.init(c.w3, holder_address)
-    txf = TokenTxFactory()
+    chain_spec = CICRegistry.chain_spec
+    TokenTxFactory.init(c.w3, chain_spec) #, holder_address)
+    txf = TokenTxFactory(holder_address)
 
     tx_transfer = txf.transfer(t['address'], receiver_address, value)
     (tx_hash_hex, tx_signed_raw_hex) = sign_and_register_tx(tx_transfer)
@@ -203,16 +189,75 @@ def transfer(self, tokens, holder_address, receiver_address, value):
     s_otx_cache.apply_async()
 
     s = create_check_gas_and_send_task(
-             [tx_hash_hex],
              [tx_signed_raw_hex],
              holder_address,
              gas_budget,
+             [tx_hash_hex],
              queue,
             )
     s.apply_async()
     return tx_hash_hex
 
 
+@celery_app.task()
+def resolve_tokens_by_symbol(token_symbols):
+    """Returns contract addresses of an array of ERC20 token symbols
+
+    :param token_symbols: Token symbols to resolve
+    :type token_symbols: list of str
+
+    :return: Respective token contract addresses
+    :rtype: list of str, 0x-hex
+    """
+    tokens = []
+    chain_spec = CICRegistry.chain_spec
+    for token_symbol in token_symbols:
+        token = CICRegistry.get_token(chain_spec, token_symbol)
+        tokens.append({
+            'address': token.address(),
+            #'converters': [],
+            })
+    return tokens
+
+#
+#@celery_app.task(bind=True)
+#def resolve_tokens_by_symbol(token_symbols):
+#    """Returns contract addresses of an array of ERC20 token symbols
+#
+#    :param token_symbols: Token symbols to resolve
+#    :type token_symbols: list of str
+#
+#    :return: Respective token contract addresses
+#    :rtype: list of str, 0x-hex
+#    """
+#
+#    queue = self.request.delivery_info['routing_key']
+#
+#    tokens = []
+#    chain_spec = CICRegistry.chain_spec
+#    fn = None
+#    try:
+#        c = CICRegistry.get_contract(CICRegistry.chain_spec, 'TokenRegistry')
+#        fn = c.function('tokenSymbolIndex')
+#    except:
+#        s = celery.signature(
+#            'cic_eth.eth.bancor.resolve_bancor_tokens_by_symbol',
+#            [token_symbols],
+#            queue=queue,
+#            )
+#        s.apply_async()
+#        return
+#
+#    for token_symbol in token_symbols:
+#        #token = CICRegistry.get_token(chain_spec, token_symbol)
+#        token = fn(token_symbol)
+#        tokens.append({
+#            'address': token.address(),
+#            'converters': [],
+#            })
+#    return tokens
+
+
 @celery_app.task()
 def otx_cache_transfer(
         tx_hash_hex,
@@ -220,7 +265,7 @@ def otx_cache_transfer(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, ContractRegistry.chain_id)
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
     (txc, cache_id) = cache_transfer_data(tx_hash_hex, tx, True)
     return txc
 
