@@ -9,16 +9,20 @@ import web3
 # local imports
 from .rpc import RpcClient
 from cic_eth.db import Otx, SessionBase
+from cic_eth.db.models.tx import TxCache
 from cic_eth.error import PermanentTxError, TemporaryTxError
-from cic_registry import ContractRegistry
+from cic_registry import CICRegistry
+from cic_registry import zero_address
 from cic_eth.queue.tx import create as queue_create
 from cic_eth.queue.tx import get_tx
 from cic_eth.error import OutOfGasError
 from cic_eth.eth.util import unpack_signed_raw_tx
 from cic_eth.eth.task import sign_and_register_tx, create_check_gas_and_send_task
+from cic_eth.eth.rpc import NonceOracle
+from cic_eth.error import AlreadyFillingGasError
 
 celery_app = celery.current_app
-#logg = celery_app.log.get_default_logger()
+loggg = celery_app.log.get_default_logger()
 logg = logging.getLogger()
 
 
@@ -53,6 +57,7 @@ def check_gas(self, tx_hashes, txs=[], address=None, gas_required=None):
 
     c = RpcClient(address)
     balance = c.w3.eth.getBalance(address) 
+    logg.debug('check gas txs {}'.format(tx_hashes))
     logg.debug('address {} has gas {} needs {}'.format(address, balance, gas_required))
 
     logg.warning('{} {}'.format(gas_required, balance))
@@ -64,7 +69,18 @@ def check_gas(self, tx_hashes, txs=[], address=None, gas_required=None):
                 ],
             queue=queue,
                 )
+        wait_tasks = []
         s_refill_gas.apply_async()
+        for tx_hash in tx_hashes:
+            s = celery.signature(
+                'cic_eth.queue.tx.set_waitforgas',
+                [
+                    tx_hash,
+                    ],
+                queue=queue,
+                )
+            wait_tasks.append(s)
+        celery.group(wait_tasks)()
         raise OutOfGasError('need to fill gas, required {}, had {}'.format(gas_required, balance))
 
     safe_gas = c.safe_threshold_amount()
@@ -82,6 +98,32 @@ def check_gas(self, tx_hashes, txs=[], address=None, gas_required=None):
     return txs
 
 
+@celery_app.task(bind=True)
+def hashes_to_txs(self, tx_hashes):
+    #logg = celery_app.log.get_default_logger()
+    if len(tx_hashes) == 0:
+        raise ValueError('no transaction to send')
+
+    queue = self.request.delivery_info['routing_key']
+
+    #otxs = ','.format("'{}'".format(tx_hash) for tx_hash in tx_hashes)
+
+    session = SessionBase.create_session()
+    q = session.query(Otx.signed_tx)
+    q = q.filter(Otx.tx_hash.in_(tx_hashes))
+    tx_tuples = q.all()
+    session.close()
+
+    def __head(x):
+        return x[0]
+
+    txs = []
+    for f in map(__head, tx_tuples):
+        txs.append(f)
+
+    return txs
+
+
 @celery_app.task(bind=True)
 def send(self, txs):
     """Send transactions to the network.
@@ -103,7 +145,7 @@ def send(self, txs):
     :return: transaction hash of sent transaction
     :rtype: str, 0x-hex
     """
-    logg = celery_app.log.get_default_logger()
+    #logg = celery_app.log.get_default_logger()
     if len(txs) == 0:
         raise ValueError('no transaction to send')
 
@@ -173,12 +215,25 @@ def refill_gas(self, recipient_address):
     :rtype: str, 0x-hex
     """
 
+    session = SessionBase.create_session()
+    q = session.query(Otx.tx_hash)
+    q = q.join(TxCache)
+    q = q.filter(Otx.status<=0)
+    q = q.filter(TxCache.from_value>0)
+    c = q.count()
+    logg.debug('q {} {}'.format(q, c))
+    session.close()
+    if c > 0:
+        raise AlreadyFillingGasError(recipient_address)
+
     queue = self.request.delivery_info['routing_key']
 
     c = RpcClient()
     clogg = celery_app.log.get_default_logger()
     logg.debug('refill gas from provider address {}'.format(c.gas_provider()))
-    nonce = c.w3.eth.getTransactionCount(c.gas_provider(), 'pending')
+    default_nonce = c.w3.eth.getTransactionCount(c.gas_provider(), 'pending')
+    nonce_generator = NonceOracle(c.gas_provider(), default_nonce)
+    nonce = nonce_generator.next()
     gas_price = c.gas_price()
     # defaultgas doesn't seem to work with ganache
     #gas_limit = RpcClient.w3.eth.defaultGas
@@ -193,7 +248,7 @@ def refill_gas(self, recipient_address):
                 'to': recipient_address,
                 'gas': gas_limit,
                 'gasPrice': gas_price,
-                'chainId': ContractRegistry.chain_id,
+                'chainId': CICRegistry.chain_spec.chain_id(),
                 'nonce': nonce,
                 'value': refill_amount,
                 'data': '',
@@ -205,21 +260,67 @@ def refill_gas(self, recipient_address):
 
     q_tx = queue_create(
         nonce,
-        recipient_address,
+        c.gas_provider(),
+        #recipient_address,
         tx_hash_hex,
         tx_send_gas_signed['raw'],   
         )
     logg.debug('added queue refill gas tx {}'.format(q_tx))
 
-    s_send = celery.signature(
-            'cic_eth.eth.tx.send',
-            [[tx_send_gas_signed['raw']]],
+    s_tx_cache = celery.signature(
+            'cic_eth.eth.tx.cache_gas_refill_data',
+            [
+                tx_hash_hex,
+                tx_send_gas,
+                ],
             queue=queue,
             )
-    s_send.apply_async()
-    return tx_hash_hex
+#    s_send = celery.signature(
+#            'cic_eth.eth.tx.send',
+#            [
+#                [tx_send_gas_signed['raw']]
+#                ],
+#            queue=queue,
+#            )
+#    celery.group([s_tx_cache, s_send])()
+    s_tx_cache.apply_async()
+    #return tx_hash_hex
+    return tx_send_gas_signed['raw']
 
 
+#@celery_app.task(bind=True)
+#def retry(self, sender_address, otx_status):
+#    queue = self.request.delivery_info['routing_key']
+#
+#    chain_id = CICRegistry.chain_spec.chain_id()
+#
+#    session = SessionBase.create_session()
+#    otxs = session.query(Otx).filter(Otx.status==otx_status).all()
+#    tx_hashes_hex = []
+#    tx_signed_raws_hex = []
+#
+#    gas = 0
+#
+#    for otx in otxs:
+#        tx_signed_bytes = bytes.fromhex(otx.signed_tx[2:])
+#        tx = unpack_signed_raw_tx(tx_signed_bytes, chain_id)
+#        if tx['from'] != sender_address:
+#            continue
+#        tx_hashes_hex.append(otx.tx_hash)
+#        tx_signed_raws_hex.append(otx.signed_tx)
+#        gas += tx['gas'] * tx['gasPrice']
+#
+#    s = create_check_gas_and_send_task(
+#        tx_signed_raws_hex,
+#        sender_address,
+#        gas,
+#        tx_hashes_hex,
+#        queue,
+#        )
+#    s.apply_async()
+#    return tx_hashes_hex
+#
+
 @celery_app.task(bind=True)
 def resend_with_higher_gas(self, txold_hash_hex, gas=None, default_factor=1.1):
     session = SessionBase.create_session()
@@ -229,7 +330,7 @@ def resend_with_higher_gas(self, txold_hash_hex, gas=None, default_factor=1.1):
     session.close()
 
     tx_signed_raw_bytes = bytes.fromhex(otx.signed_tx[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, ContractRegistry.chain_id)
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
     logg.debug('otx {} {}'.format(tx, otx.signed_tx))
 
     queue = self.request.delivery_info['routing_key']
@@ -249,11 +350,70 @@ def resend_with_higher_gas(self, txold_hash_hex, gas=None, default_factor=1.1):
 
     (tx_hash_hex, tx_signed_raw_hex) = sign_and_register_tx(tx)
     s = create_check_gas_and_send_task(
-            [tx_hash_hex],
             [tx_signed_raw_hex],
             tx['from'],
             tx['gasPrice'] * tx['gas'],
+            [tx_hash_hex],
             queue=queue,
             )
     s.apply_async()
     return tx_hash_hex
+
+
+@celery_app.task(bind=True)
+def resume_tx(self, txpending_hash_hex):
+    session = SessionBase.create_session()
+    tx_signed_raw_hex = session.query(Otx.signed_tx).filter(Otx.tx_hash==txold_hash_hex).first()
+    session.close()
+    if otx == None:
+        raise NotLocalTxError(txold_hash_hex)
+
+    queue = self.request.delivery_info['routing_key']
+
+    s = create_check_gas_and_send_task(
+            [tx_signed_raw_hex],
+            tx['from'],
+            tx['gasPrice'] * tx['gas'],
+            [txpending_hash_hex],
+            queue=queue,
+            )
+    s.apply_async()
+    return txpending_hash_hex
+
+
+# TODO: connect to refill gas task
+@celery_app.task()
+def otx_cache_parse_tx(
+        tx_hash_hex,
+        tx_signed_raw_hex,
+        ):
+
+    tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    (txc, cache_id) = cache_gas_refill_data(tx_hash_hex, tx)
+    return txc
+
+
+# TODO: connect to refill gas task
+@celery_app.task()
+def cache_gas_refill_data(
+    tx_hash_hex,
+    tx,
+        ):
+
+    tx_cache = TxCache(
+        tx_hash_hex,
+        tx['from'],
+        tx['to'],
+        zero_address,
+        zero_address,
+        tx['value'],
+        tx['value'],
+            )
+
+    session = SessionBase.create_session()
+    session.add(tx_cache)
+    session.commit()
+    cache_id = tx_cache.id
+    session.close()
+    return (tx_hash_hex, cache_id)
