@@ -9,9 +9,9 @@ class HeadSyncer(MinedSyncer):
     :param bc_cache: Retrieves block cache cursors for chain head and latest processed block.
     :type bc_cache: object implementing methods head(height) and backlog(height)
     """
-    def __init__(self, w3, bc_cache):
-        super(HeadSyncer, self).__init__(bc_cache)
-        self.w3_filter = w3.eth.filter('latest')
+    def __init__(self, rpc, bc_cache):
+        super(HeadSyncer, self).__init__(rpc, bc_cache)
+        self.w3_filter = rpc.w3.eth.filter('latest')
    
 
     """Implements Syncer.get
