@@ -20,6 +20,13 @@ class MinedSyncer(Syncer):
     :param block_hash_hex: Block hash
     :type block_hash_hex: str, 0x-hash
     """
+
+    def __init__(self, rpc, bc_cache):
+        super(MinedSyncer, self).__init__(bc_cache)
+        self.rpc = rpc
+        self.w3 = rpc.w3
+
+
     def process(self, w3, block_hash_hex):
         logg.debug('processing {}'.format(block_hash_hex))
         b = w3.eth.getBlock(block_hash_hex)
@@ -29,15 +36,21 @@ class MinedSyncer(Syncer):
             tx_hash_hex = tx['hash'].hex()
             rcpt = w3.eth.getTransactionReceipt(tx_hash_hex)
             logg.debug('{}/{} processing tx {} from block {}'.format(i+1, c, tx_hash_hex, block_hash_hex))
-            s = celery.signature(
-                    'set_final_status',
-                    [tx_hash_hex, rcpt['blockNumber'], not rcpt['status']],
-                    )
-            s.apply_async()
+            ours = False
             for f in self.filter:
-                #try:
-                f(w3, tx, rcpt)
-                #except Exception as e:
-                #logg.debug(e)
+                try:
+                    task_uuid = f(w3, tx, rcpt)
+                except Exception as e:
+                    logg.error('error in filter {} tx {}: {}'.format(f, tx_hash_hex, e))
+                    continue
+                if task_uuid != None:
+                    logg.debug('tx {} passed to celery task {}'.format(tx_hash_hex, task_uuid))
+                    s = celery.signature(
+                            'set_final_status',
+                            [tx_hash_hex, rcpt['blockNumber'], not rcpt['status']],
+                            )
+                    s.apply_async()
+                    break
+
         return b['number']
 
